@@ -25,6 +25,7 @@ import io
 import math
 import os
 import re
+import sys
 import warnings
 
 from flent import combiners
@@ -40,25 +41,22 @@ from itertools import cycle, islice, chain
 from collections import OrderedDict
 from distutils.version import LooseVersion
 
+logger = get_logger(__name__)
+
 try:
     import matplotlib
     import numpy as np
     HAS_MATPLOTLIB = True
     MPL_VER = LooseVersion(matplotlib.__version__)
-except ImportError:
+    if MPL_VER < LooseVersion("1.5"):
+        logger.warning("Cannot use old matplotlib version %s, please upgrade!",
+                       matplotlib.__version__)
+        raise ImportError("Matplotlib %s too old" % matplotlib.__version__)
+except ImportError as e:
+    logger.debug("Unable to import matplotlib: %s", e)
     HAS_MATPLOTLIB = False
     MPL_VER = LooseVersion("0")
 
-# Python 2/3 compatibility
-try:
-    unicode
-    PY2 = True
-except NameError:
-    unicode = str
-    PY2 = False
-
-logger = get_logger(__name__)
-
 PLOT_KWARGS = (
     'alpha',
     'antialiased',
@@ -127,28 +125,6 @@ MATPLOTLIB_STYLES = {'axes.axisbelow': True,
 
 MATPLOTLIB_INIT = False
 
-if PY2:
-    # Matplotlib will tend to pass the values directly to backends where they
-    # will be converted into native types. This breaks on some versions of
-    # matplotlib running on Python 2 if the values are unicode objects (which
-    # can't always be automatically converted). To work around this, encode
-    # everything if we are running on Python 2.
-
-    def filt(x):
-        try:
-            return x.encode()
-        except AttributeError:
-            return x
-
-    LINESTYLES = list(map(filt, LINESTYLES))
-    MARKERS = list(map(filt, MARKERS))
-    COLOURS = list(map(filt, COLOURS))
-
-    for k, v in MATPLOTLIB_STYLES.items():
-        MATPLOTLIB_STYLES[k] = filt(v)
-
-    del filt
-
 
 def init_matplotlib(output, use_markers, load_rc):
     if not HAS_MATPLOTLIB:
@@ -176,7 +152,7 @@ def init_matplotlib(output, use_markers, load_rc):
             raise RuntimeError(
                 "Unrecognised file format for output '%s'" % output)
 
-    elif not os.getenv("DISPLAY"):
+    elif (sys.platform == 'linux' and not os.getenv("DISPLAY")):
         matplotlib.use("agg")
 
     from matplotlib import pyplot
@@ -257,13 +233,13 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--label-x",
-        action="append", dest="LABEL_X", type=unicode, default=[],
+        action="append", dest="LABEL_X", type=str, default=[],
         help="Override X axis labels. "
         "Can be specified twice, corresponding to figures with multiple axes.")
 
     parser.add_argument(
         "--label-y",
-        action="append", dest="LABEL_Y", type=unicode, default=[],
+        action="append", dest="LABEL_Y", type=str, default=[],
         help="Override Y axis labels. "
         "Can be specified twice, corresponding to figures with multiple axes.")
 
@@ -285,7 +261,7 @@ def add_plotting_args(parser):
                         const="log10", help=SUPPRESS)
     parser.add_argument(
         "--log-scale-y",
-        action="store", type=unicode, dest="LOG_SCALE", choices=("log2", "log10"),
+        action="store", type=str, dest="LOG_SCALE", choices=("log2", "log10"),
         help="Use logarithmic scale.")
 
     parser.add_argument(
@@ -372,7 +348,7 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--figure-note", "--fig-note",
-        action="store", type=unicode, dest="FIG_NOTE",
+        action="store", type=str, dest="FIG_NOTE",
         help="Figure note. Will be added to the bottom-left corner of the "
         "figure.")
 
@@ -383,7 +359,7 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--override-title",
-        action="store", type=unicode, dest="OVERRIDE_TITLE", metavar="TITLE",
+        action="store", type=str, dest="OVERRIDE_TITLE", metavar="TITLE",
         help="Override plot title. This parameter takes "
         "precedence over --no-title.")
 
@@ -417,12 +393,12 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--legend-title",
-        action="store", type=unicode, dest="LEGEND_TITLE",
+        action="store", type=str, dest="LEGEND_TITLE",
         help="Override legend title.")
 
     parser.add_argument(
         "--legend-placement",
-        action="store", type=unicode, dest="LEGEND_PLACEMENT",
+        action="store", type=str, dest="LEGEND_PLACEMENT",
         choices=('best',
                  'upper right',
                  'upper left',
@@ -464,27 +440,27 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--filter-regexp",
-        action="append", type=unicode, dest="FILTER_REGEXP", metavar="REGEXP",
-        default=[], help="Filter labels (regex). Filter out supplied regular "
-        "expression from label names. Can be specified multiple times, in which "
-        "case the regular expressions will be filtered in the order specified.")
+        action="append", type=str, dest="FILTER_REGEXP", metavar="REGEXP",
+        default=[], help="Filter labels (regex). Filter out (remove) text matching "
+        "the supplied regular expression from label names. Can be specified multiple "
+        "times, in which case the filters will be applied in the order specified.")
 
     parser.add_argument(
         "--override-label",
-        action="append", type=unicode, dest="OVERRIDE_LABELS", metavar="LABEL",
+        action="append", type=str, dest="OVERRIDE_LABELS", metavar="LABEL",
         default=[],
         help="Override dataset labels. Must be specified multiple times "
         "corresponding to the datasets being overridden.")
 
     parser.add_argument(
         "--filter-series",
-        action="append", type=unicode, dest="FILTER_SERIES", metavar="SERIES",
+        action="append", type=str, dest="FILTER_SERIES", metavar="SERIES",
         default=[], help="Filter (hide) data series. Filters out specified "
         "series names from the plot. Can be specified multiple times.")
 
     parser.add_argument(
         "--split-group",
-        action="append", type=unicode, dest="SPLIT_GROUPS", default=[],
+        action="append", type=str, dest="SPLIT_GROUPS", default=[],
         metavar="LABEL",
         help="New groups for box and bar plots. Specify this option multiple "
         "times to define the new groups that data sets should be split into on "
@@ -499,7 +475,7 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--override-colour-mode",
-        action="store", type=unicode, dest="OVERRIDE_COLOUR_MODE", metavar="MODE",
+        action="store", type=str, dest="OVERRIDE_COLOUR_MODE", metavar="MODE",
         help="Override colour_mode attribute. This changes the way colours "
         "are assigned to bar plots. The default is 'groups' which assigns a "
         "separate colour to each group of data series. The alternative is "
@@ -508,14 +484,14 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--override-group-by",
-        action="store", type=unicode, dest="OVERRIDE_GROUP_BY", metavar="GROUP",
+        action="store", type=str, dest="OVERRIDE_GROUP_BY", metavar="GROUP",
         help="Override group_by attribute. This changes the way combination "
         "plots are created by overriding the function that is used to combine "
         "several data series into one.")
 
     hide_gui(parser.add_argument(
         "--combine-save-dir",
-        action="store", type=unicode, dest="COMBINE_SAVE_DIR",
+        action="store", type=str, dest="COMBINE_SAVE_DIR",
         metavar="DIRNAME",
         help="Save intermediate combination data. When doing a combination plot "
         "save the intermediate data to DIRNAME. This can then be used for "
@@ -564,7 +540,7 @@ def add_plotting_args(parser):
 
     hide_gui(parser.add_argument(
         "--scale-data",
-        action="append", type=unicode, dest="SCALE_DATA", default=[],
+        action="append", type=str, dest="SCALE_DATA", default=[],
         help="Extra scale data. Additional data files to consider when scaling "
         "the plot axes "
         "(for plotting several plots with identical axes). Note, this displays "
@@ -680,7 +656,7 @@ class Plotter(ArgParam):
             self.figure._original_dpi = self.figure.dpi
 
     def __del__(self):
-        if not self.disable_cleanup:
+        if not getattr(self, "disable_cleanup", False):
             try:
                 pyplot.close(self.figure)
             except Exception:
@@ -688,20 +664,10 @@ class Plotter(ArgParam):
 
     def __getstate__(self):
         state = self.__dict__.copy()
-
-        for c in state['configs']:
-            for a in c['axes']:
-                # There is a bug in matplotlib prior to 1.4.2 where the renderer
-                # can't be pickled. Work around this by monkey-patching the
-                # renderer object as needed.
-                if getattr(a, "_cachedRenderer", None) \
-                   and not hasattr(a._cachedRenderer, "__getstate__"):
-                    def getState():
-                        return {'width': a._cachedRenderer.width,
-                                'height': a._cachedRenderer.height,
-                                'dpi': a._cachedRenderer.dpi}
-                    a._cachedRenderer.__getstate__ = getState
-
+        if 'figure' in state:
+            fig = state['figure']
+            if getattr(fig, "canvas", None) and getattr(fig.canvas, "manager", None):
+                fig.canvas.manager = None
         return state
 
     def init(self, config=None, axis=None):
@@ -1073,11 +1039,17 @@ class Plotter(ArgParam):
            and self.legends:
 
             # Make sure we have a renderer to get size from
-            if not getattr(self.figure, '_cachedRenderer', None):
+            renderer = getattr(self.figure, '_cachedRenderer', None)
+            if not renderer:
                 self.figure.canvas.draw()
+                renderer = getattr(self.figure, '_cachedRenderer', None)
 
-            legend_width = max(
-                [l.get_window_extent().width for l in self.legends])
+            try:
+                legend_width = max(
+                    [l.get_window_extent(renderer).width for l in self.legends])
+            except Exception as e:
+                logger.debug("Error getting legend sizes: %s", e)
+                return
 
             canvas_width = self.figure.canvas.get_width_height()[0]
             for a in self.axes_iter():
@@ -1310,8 +1282,8 @@ class Plotter(ArgParam):
             if end <= 0:
                 end += results.meta("TOTAL_LENGTH")
 
-            min_idx = data[0].searchsorted(start, side='right')
-            max_idx = data[0].searchsorted(end, side='left')
+            min_idx = data[0].searchsorted(start, side='left')
+            max_idx = data[0].searchsorted(end, side='right')
 
             data = data[:, min_idx:max_idx]
 
@@ -1401,7 +1373,7 @@ class TimeseriesPlotter(Plotter):
             else:
                 a = 0
             s_unit = self.data_config[s['data']]['units']
-            if unit[a] is not None and s_unit != unit[a]:
+            if unit[a] is not None and s_unit != unit[a] and 'raw_key' not in s:
                 raise RuntimeError(
                     "Plot axis unit mismatch: %s/%s" % (unit[a], s_unit))
             unit[a] = s_unit
@@ -1802,7 +1774,6 @@ class BarPlotter(BoxPlotter):
             if len(l) > self._max_label_length:
                 ticklabels[i] = l[:self._max_label_length] + "..."
 
-        axis.set_xticklabels(ticklabels, rotation=90, ha='center')
         axis.set_xlim(0, pos - 1)
         axis.set_xticks(ticks)
         axis.set_xticks([], minor=True)
