@@ -39,7 +39,7 @@ from flent.build_info import VERSION
 from flent.testenv import TestEnvironment, TEST_PATH
 from flent.loggers import get_logger
 from flent.util import FuncAction, Update, AddHost, append_host, keyval, \
-    keyval_int, keyval_transformer, ArgParser, token_split
+    keyval_int, keyval_transformer, ArgParser, token_split, global_cache
 from flent.plotters import add_plotting_args
 from flent import loggers, util, resultset, runners
 
@@ -378,6 +378,12 @@ test_group.add_argument(
     "to decrease the socket buffer size. Can be specified multiple times, "
     "with each value corresponding to a stream of a test.")
 
+test_group.add_argument(
+    "--test-payload",
+    action="store", type=unicode, dest="TEST_PAYLOAD", default='/dev/urandom',
+    help="Path to file containing payload to pre-fill the netperf buffers with "
+    "defaults to randomised payload")
+
 test_group.add_argument(
     "--test-parameter",
     action=Update, type=keyval, dest="TEST_PARAMETERS", metavar='key=value',
@@ -485,6 +491,14 @@ tool_group.add_argument(
     default=4, help="Number of workers to use for getting HTTP urls. "
     "Default is 4.")
 
+tool_group.add_argument(
+    "--irtt-sampling-interval",
+    action="store", type=int, dest="IRTT_INTERVAL", metavar="MILLISECONDS",
+    help="Override the sampling interval passed to irtt, in milliseconds. Can be "
+    "used to run irtt with a higher sampling frequency than the rest of the test. "
+    "If set, this will override the sampling interval for all instances of irtt "
+    "used in the test.")
+
 misc_group = parser.add_argument_group("Misc and debugging options")
 
 misc_group.add_argument(
@@ -494,6 +508,12 @@ misc_group.add_argument(
     "enabled but no file name is given, the log file name is derived from the "
     "test data filename.")
 
+misc_group.add_argument(
+    "--cache-file",
+    action="store", type=unicode, dest="CACHE_FILE", metavar="FILENAME",
+    help="Use this file for caching lookup of runner file names and "
+    "versions between runs.")
+
 misc_group.add_argument(
     '--list-tests',
     action=ListTests,
@@ -740,6 +760,9 @@ class Settings(argparse.Namespace):
         elif self.NEW_GUI_INSTANCE:
             self.GUI = True
 
+        if self.CACHE_FILE:
+            global_cache.read_file(self.CACHE_FILE)
+
         if self.REMOTE_METADATA:
             self.EXTENDED_METADATA = True
 
