@@ -22,11 +22,16 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import argparse
+import fcntl
 import fnmatch
+import ipaddress
+import json
 import os
 import re
 import shlex
+import shutil
 import socket
+import threading
 import time
 import subprocess
 
@@ -176,42 +181,175 @@ def diff_parts(strings, sep):
 
     return [sep.join(p) for p in zip(*np)]
 
+class CachingDictionary(dict):
 
-def is_executable(filename):
-    return os.path.isfile(filename) and os.access(filename, os.X_OK)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.filename = None
+        self.lock = threading.Lock()
+
+    def read_file(self, filename):
+        try:
+            with open(filename) as fp:
+                fcntl.flock(fp, fcntl.LOCK_EX)
+                if fp.read(1) == '':
+                    # JSON parser chokes on an empty file, so turn this into a
+                    # FileNotFoundError (which is handled below)
+                    raise FileNotFoundError
+
+                fp.seek(0)
+                obj = json.load(fp)
+                self.update(obj)
+                logger.debug("Read cache '%s' from file '%s'", self, filename)
+        except FileNotFoundError:
+            if not self.write_file(filename):
+                # File doesn't exist (or is empty), make sure we can write to it
+                logger.warning("Couldn't write to cache file '%s'; not using.",
+                               filename)
+                return
+            else:
+                logger.debug("Created cache file at '%s'", filename)
+        except (IOError, json.decoder.JSONDecodeError) as e:
+            logger.warning("Error reading cache file '%s' (%s); not using.",
+                           self.filename, e)
+            return
+
+        self.filename = filename
+
+    def write_file(self, filename=None):
+        if filename is None:
+            filename = self.filename
+
+        if filename is None:
+            return
+
+        try:
+            with open(filename, 'w') as fp:
+                fcntl.flock(fp, fcntl.LOCK_EX)
+                json.dump(self, fp)
+                return True
+        except IOError as e:
+            logger.debug("Writing cache file failed: '%s'", e)
+            return False
+
+    def __setitem__(self, key, value):
+        with self.lock:
+            super().__setitem__(key, value)
+            self.write_file()
+
+    def get_view(self, key):
+        return CachingDictView(self, key)
 
 
-def which(executable, fail=None, remote_host=None):
-    pathname, filename = os.path.split(executable)
-    if pathname:
-        if is_executable(executable):
-            logger.debug("which: %s is a full path and executable", executable)
-            return executable
-    elif remote_host:
-        logger.debug("running 'which' for binary '%s' on host '%s'",
+class CachingDictView:
+
+    def __init__(self, parent, key):
+        if key not in parent:
+            parent[key] = {}
+        self.parent = parent
+        self.key = key
+
+    def __getitem__(self, key):
+        return self.parent[self.key][key]
+
+    def __setitem__(self, key, value):
+        with self.parent.lock:
+            self.parent[self.key][key] = value
+            self.parent.write_file()
+
+    def __delitem__(self, key):
+        with self.parent.lock:
+            del self.parent[self.key][key]
+            self.parent.write_file()
+
+    def __contains__(self, key):
+        return key in self.parent[self.key]
+
+    def __iter__(self):
+        return iter(self.parent[self.key])
+
+    def update(self, *args, **kwargs):
+        with self.parent.lock:
+            self.parent[self.key].update(*args, **kwargs)
+            self.parent.write_file()
+
+    def keys(self):
+        return self.parent[self.key].keys()
+
+    def values(self):
+        return self.parent[self.key].values()
+
+    def items(self):
+        return self.parent[self.key].items()
+
+global_cache = CachingDictionary()
+get_cache = global_cache.get_view
+
+
+class WhichCache:
+
+    def __init__(self):
+        self.cache = get_cache("which")
+
+    @staticmethod
+    def is_executable(filename):
+        return os.path.isfile(filename) and os.access(filename, os.X_OK)
+
+    @staticmethod
+    def lookup_remote(executable, remote_host):
+        logger.debug("running 'command -v' for binary '%s' on host '%s'",
                      executable, remote_host)
         try:
             output = subprocess.check_output(['ssh', remote_host,
-                                              'which {}'.format(executable)],
+                                              'command -v {}'.format(executable)],
                                              timeout=1)
             output = output.decode(ENCODING).strip()
-            logger.debug("Got path '%s' for '%s' on '%s'", output, executable, remote_host)
+            logger.debug("Got path '%s' for '%s' on '%s'",
+                         output, executable, remote_host)
             return output
         except subprocess.CalledProcessError:
             pass
-    else:
-        for path in [i.strip('""') for i in os.environ["PATH"].split(os.pathsep)]:
-            filename = os.path.join(path, executable)
-            if is_executable(filename):
-                logger.debug("which: Found %s executable at %s",
-                             executable, filename)
-                return filename
-            else:
-                logger.debug("which: %s is not an executable file", filename)
 
-    if fail:
-        raise fail("No %s binary found in PATH." % executable)
-    return None
+        return None
+
+    def lookup_local(self, executable):
+        pathname, filename = os.path.split(executable)
+        if pathname:
+            if not self.is_executable(executable):
+                logger.debug("which: %s not executable", executable)
+                return None
+
+            logger.debug("which: %s is a full path and executable", executable)
+            return executable
+
+        filename = shutil.which(executable)
+        if filename is not None:
+            logger.debug("which: Found %s executable at %s",
+                         executable, filename)
+            return filename
+
+        return None
+
+    def __call__(self, executable, fail=None, remote_host=None):
+        key = f"{executable},{remote_host or ''}"
+        if key in self.cache:
+            logger.debug("which: found '%s' in cache: %s", key, self.cache[key])
+            return self.cache[key]
+
+        if remote_host:
+            binary = self.lookup_remote(executable, remote_host)
+        else:
+            binary = self.lookup_local(executable)
+
+        if not binary and fail:
+            raise fail("No %s binary found in PATH." % executable)
+
+        if binary:
+            self.cache[key] = binary
+        return binary
+
+# callers call 'util.which', so make that an instance of the cache
+which = WhichCache()
 
 
 def path_components(path):
@@ -225,9 +363,15 @@ def path_components(path):
         folders.insert(0, path)
     return folders
 
-def normalise_host(hostname):
+def normalise_host(hostname, bracket_v6=False):
     if hostname and MULTIHOST_SEP in hostname:
-        return hostname.split(MULTIHOST_SEP, 1)[0]
+        hostname = hostname.split(MULTIHOST_SEP, 1)[0]
+    if bracket_v6:
+        try:
+            ipaddress.IPv6Address(hostname)
+            hostname = "[{}]".format(hostname)
+        except ValueError:
+            pass
     return hostname
 
 def lookup_host(hostname, version=None):
@@ -313,10 +457,9 @@ class Glob(object):
             pattern = self.pattern
 
         # Exclude * from matching :, make ** match everything
-        re_pat = fnmatch.translate(pattern)
-        re_pat = re_pat.replace(".*.*", ":::PLACEHOLDER:::")
+        re_pat = fnmatch.translate(pattern.replace("**", "___PLACEHOLDER___"))
         re_pat = re_pat.replace(".*", "[^:]*")
-        re_pat = re_pat.replace(":::PLACEHOLDER:::", ".*")
+        re_pat = re_pat.replace("___PLACEHOLDER___", ".*")
         regex = re.compile(re_pat)
 
         exclude += self.exclude
@@ -361,7 +504,7 @@ def mos_score(T, loss):
     Based on ITU G.107 06/2015.
 
     Verified against the online reference implementation at
-    https://www.itu.int/ITU-T/studygroups/com12/emodelv1/calcul.php
+    https://www.itu.int/ITU-T/studygroups/com12/emodel/calculnb.php
 
     This version assumes the default values are used for all parameters other
     than delay and loss.
@@ -419,7 +562,7 @@ def mos_score(T, loss):
 
     Id = Idte + Idle + Idd  # (7-18)
 
-    Ieeff = 95 * (Ppl / (Ppl + 1))  # (7-29) with BurstR = Bpl = 1
+    Ieeff = 95 * (Ppl / (Ppl + 4.3))  # (7-29) with BurstR = 1, Bpl = 4.3
 
     R = Ro - Is - Id - Ieeff
 
